/* main.c - Application main entry point */

/*
 * Copyright (c) 2015-2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <sys/errno.h>
#include <zephyr/types.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/byteorder.h>
#include <zephyr/kernel.h>

#include <zephyr/settings/settings.h>

#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/hci.h>
#include <zephyr/bluetooth/conn.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>

#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/sensor.h>
#include <zephyr/drivers/gpio.h>

#include <math.h>

#include <zephyr/logging/log.h>
LOG_MODULE_REGISTER(main);

#include "battery.h"
#include "sht4x.h"
#include "bmp180.h"
#include "sds011.h"
#include "hal/nrf_gpiote.h"

static K_FIFO_DEFINE(fifo_adv_data);

static K_SEM_DEFINE(ble_init_ok, 0, 1);
static K_SEM_DEFINE(adv_send_ok, 0, 1);

#define BTHOME_MAXELT 9
enum captor_type { BATTERY, BATTERYVDD, TEMPERATURE, PRESSURE, VOLTAGE, VOLTAGEVDD, PM25, PM10, HUMIDITY };
uint8_t eltid[BTHOME_MAXELT] = { 0x01, 0x01, 0x02, 0x04, 0x0c, 0x0c, 0x0d, 0x0e, 0x2e };
uint8_t eltsize[BTHOME_MAXELT] = { 1, 1, 2, 3, 2, 2, 2, 2, 1};
uint8_t eltenabled[BTHOME_MAXELT] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };

struct bthome_adv_t {
    void *fifo_reserved;
    uint8_t type;
    int16_t data_s16;
    uint16_t data_u16;
    uint8_t data_u8;
    uint32_t data_u32;
};

   static uint8_t mfg_bthome_data[28];
   uint8_t mfg_bthome_sizetosend = 0;
   uint8_t mfg_bthome_numelt = 0;
   uint8_t bthome_datapos[BTHOME_MAXELT];
    //uint8_t bthome_datapos[] = { 4, 6, 9, 12, 14, 18, 21 };

 

    struct captor_wait {
        enum captor_type captor;
        uint16_t wait;
    };

    struct captor_wait params[] = { {BATTERY, 120 }, {TEMPERATURE, 60}, {PM25, 60} };


    /*static const struct bt_data ad[] = {
        BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
        BT_DATA(BT_DATA_SVC_DATA16, mfg_bthome_data, 23),
    };*/

#define GPIO_PORT DT_NODELABEL(gpio0)

#define STACKSIZE 2048
#define PRIORITY 7

K_THREAD_STACK_DEFINE(stack_wq_pms, STACKSIZE);
K_THREAD_STACK_DEFINE(stack_wq_i2c, STACKSIZE);

struct k_work_q workqueue_pms;
struct k_work_q workqueue_i2c;

    static const struct battery_level_point battery_levels[] = {
        /* "Curve" here eyeballed from captured data for the [Adafruit
         * 3.7v 2000 mAh](https://www.adafruit.com/product/2011) LIPO
         * under full load that started with a charge of 3.96 V and
         * dropped about linearly to 3.58 V over 15 hours.  It then
         * dropped rapidly to 3.10 V over one hour, at which point it
         * stopped transmitting.
         *
         * Based on eyeball comparisons we'll say that 15/16 of life
         * goes between 3.95 and 3.55 V, and 1/16 goes between 3.55 V
         * and 3.1 V.
         */

        { 10000, 3950 },
        { 625, 3550 },
        { 0, 3100 },
    };
    static const struct battery_level_point battery_nimh_levels[] = {
        /* "Curve" here eyeballed from captured data for the [Adafruit
         * 3.7v 2000 mAh](https://www.adafruit.com/product/2011) LIPO
         * under full load that started with a charge of 3.96 V and
         * dropped about linearly to 3.58 V over 15 hours.  It then
         * dropped rapidly to 3.10 V over one hour, at which point it
         * stopped transmitting.
         *
         * Based on eyeball comparisons we'll say that 15/16 of life
         * goes between 3.95 and 3.55 V, and 1/16 goes between 3.55 V
         * and 3.1 V.
         */

        { 10000, 2800 },
        { 90, 2600 },
        { 50, 2440 },
        { 0, 2300 },
    };


    
    //#define UART_DEVICE_NODE DT_CHOSEN(zephyr_shell_uart)


    /* static void connected(struct bt_conn *conn, uint8_t err)
       {
       if (err) {
       printk("Connection failed (err 0x%02x)\n", err);
       } else {
       printk("Connected\n");
       }
       }

       static void disconnected(struct bt_conn *conn, uint8_t reason)
       {
       printk("Disconnected (reason 0x%02x)\n", reason);
       }*/

  /*  uint32_t getTs()
    {e
        uint32_t ts = (k_uptime_get_32()/1000);
        // LOG_INF("getTs: %d", ts);
        return ts;
    }

   uint16_t findsmallestwait()
    {
        uint16_t minval = UINT16_MAX;
        uint32_t ts = getTs();
        for(int i = 0; i < sizeof(params) / sizeof(params[0]); i++)
        {
            if(params[i].wait != 0) {
                //LOG_INF("%s %d", params[i].captor[j].captor, params[i].captor[j].wait);
                uint32_t wait = (uint32_t)params[i].wait;
                if((uint16_t)(wait-(ts % wait)) < minval) { minval = (uint16_t)(wait-(ts % wait)); }
            }
        }
        return minval;
    }

    uint16_t findiftoprobe(enum captor_type captorp, bool record)
    {
        uint32_t ts = getTs();
        //LOG_INF("findiftoprobe: %s", captorp);
        for(int i = 0; i < sizeof(params) / sizeof(params[0]); i++)
        {
            if(captorp == params[i].captor) {
                return (uint16_t)(ts % (uint32_t)params[i].wait);
            }
        }
        return UINT16_MAX;

    }
*/

    static void bt_ready(void)
    {
        int err;

        printk("Bluetooth initialized\n");

        /*if (IS_ENABLED(CONFIG_SETTINGS)) {
          settings_load();
          }*/


     struct bt_data ad[] = {
        //BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
        BT_DATA(BT_DATA_SVC_DATA16, mfg_bthome_data, mfg_bthome_sizetosend),
    };
        err = bt_le_adv_start(BT_LE_ADV_PARAM(BT_LE_ADV_OPT_USE_IDENTITY | BT_LE_ADV_OPT_USE_NAME, BT_GAP_ADV_SLOW_INT_MIN,BT_GAP_ADV_SLOW_INT_MAX, NULL), ad, ARRAY_SIZE(ad), NULL, 0);
        if (err) {
            printk("Advertising failed to start (err %d)\n", err);
            return;
        }

        printk("Advertising successfully started\n");
    }

    /* static void auth_passkey_display(struct bt_conn *conn, unsigned int passkey)
       {
       char addr[BT_ADDR_LE_STR_LEN];

       bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

       printk("Passkey for %s: %06u\n", addr, passkey);
       }

       static void auth_cancel(struct bt_conn *conn)
       {
       char addr[BT_ADDR_LE_STR_LEN];

       bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

       printk("Pairing cancelled: %s\n", addr);
       }

       static struct bt_conn_auth_cb auth_cb = {
       .passkey_display = auth_passkey_display,
       .passkey_entry = NULL,
       .cancel = auth_cancel,
       };*/

    /*bool test_bthome_data_set()
    {
        for(int i = 0; i < sizeof(bthome_datapos); i++)
        {
            if(mfg_bthome_data[bthome_datapos[i]] == 0x00)
            {
                //printk("BTHome POS %d empty\n", bthome_datapos[i]);
                return false;
            }
        }
        return true;
    }*/

/*void shift_bthome_datapos(uint8_t value)
{
  for(int i = 0; i < BTHOME_MAXELT; i++)
  {
    if(bthome_datapos[i] == 255)
    {
      bthome_datapos[i] = bthome_datapos[i] + value;
    }
  }

}
uint8_t mfg_bthome_addelt(uint8_t eltid, uint8_t eltsize)
{
  if(mfg_bthome_numelt == 0)
  {
    mfg_bthome_numelt++;
    mfg_bthome_data[3] = eltid;
    mfg_bthome_sizetosend = mfg_bthome_sizetosend + eltsize +1;
    return 3;
  } else {
    if(eltid < mfg_bthome_data[3])
    {
      mfg_bthome_sizetosend = mfg_bthome_sizetosend + eltsize +1;
      for(int i = 0; i < eltsize+1; i++)
      {
        for(int j = mfg_bthome_sizetosend; j > 2; j--)
        {
          mfg_bthome_data[j] = mfg_bthome_data[j-1];
        }
    }
      shift_bthome_datapos(eltsize+1);
      mfg_bthome_data[3] = eltid;
      return 3;
    } else {
      mfg_bthome_data[mfg_bthome_sizetosend] = eltid;
      mfg_bthome_sizetosend = mfg_bthome_sizetosend + eltsize +1;
      return mfg_bthome_sizetosend - eltsize - 1;
    }
  }
}
*/

void mfg_bthome_build()
{
  for(int i = 0; i < BTHOME_MAXELT; i++)
  {
    if(eltenabled[i] == 1)
    {
    mfg_bthome_numelt++;
      mfg_bthome_data[mfg_bthome_sizetosend] = eltid[i];
      bthome_datapos[i] = mfg_bthome_sizetosend;
      mfg_bthome_sizetosend = mfg_bthome_sizetosend + eltsize[i] + 1;
    }
  }
}

    void publish_adv_thread()
    {
        bool adv_activate = false;
        uint8_t numeltreceived = 0;
uint8_t eltreceived[BTHOME_MAXELT] = { 0, 0, 0, 0, 0, 0, 0 };
   //static uint8_t mfg_bthome_data[] = { 0xd2, 0xfc, 0x40, 0x01, 0x00, 0x0c, 0x00, 0x00, 0x02, 0x00, 0x00, 0x2e, 0x00, 0x04, 0x00, 0x00, 0x00, 0x0d, 0x00, 0x00, 0x0e, 0x00, 0x00 };
    //uint8_t bthome_datapos[] = { 4, 6, 9, 12, 14, 18, 21 };

        k_sem_take(&ble_init_ok, K_FOREVER);

        for (;;) {
    k_sem_take(&adv_send_ok, K_FOREVER);
            while(k_fifo_is_empty(&fifo_adv_data) == 0)
    {
            struct bthome_adv_t *request = k_fifo_get(&fifo_adv_data, K_NO_WAIT);
/*            if(bthome_datapos[request->type] == 255)
      {
            bthome_datapos[request->type] = mfg_bthome_addelt(eltid[request->type], eltsize[request->type]);        
      }*/
            
      if(eltreceived[request->type] == 0) {
      numeltreceived++;
      eltreceived[request->type] = 1;
      }
            if(request->type == VOLTAGE)
            {
                mfg_bthome_data[bthome_datapos[VOLTAGE]+2] = request->data_u16 >> 8;
                mfg_bthome_data[bthome_datapos[VOLTAGE]+1] = request->data_u16;
            } else if(request->type == BATTERY)
            {
                mfg_bthome_data[bthome_datapos[BATTERY]+1] = request->data_u8;
            } else if(request->type == VOLTAGEVDD)
            {
                mfg_bthome_data[bthome_datapos[VOLTAGEVDD]+2] = request->data_u16 >> 8;
                mfg_bthome_data[bthome_datapos[VOLTAGEVDD]+1] = request->data_u16;
            } else if(request->type == BATTERYVDD)
            {
                mfg_bthome_data[bthome_datapos[BATTERYVDD]+1] = request->data_u8;
            } else if(request->type == TEMPERATURE)
            {
                mfg_bthome_data[bthome_datapos[TEMPERATURE]+2] = request->data_s16 >> 8;
                mfg_bthome_data[bthome_datapos[TEMPERATURE]+1] = request->data_s16;
            } else if(request->type == HUMIDITY)
            {
                mfg_bthome_data[bthome_datapos[HUMIDITY]+1] = request->data_u8;
            } else if(request->type == PRESSURE)
            {
                mfg_bthome_data[bthome_datapos[PRESSURE]+3] = request->data_u32 >> 16;
                mfg_bthome_data[bthome_datapos[PRESSURE]+2] = request->data_u32 >> 8;
                mfg_bthome_data[bthome_datapos[PRESSURE]+1] = request->data_u32;
            } else if(request->type == PM25)
            {
                mfg_bthome_data[bthome_datapos[PM25]+2] = request->data_u16 >> 8;
                mfg_bthome_data[bthome_datapos[PM25]+1] = request->data_u16;
            } else if(request->type == PM10)
            {
                mfg_bthome_data[bthome_datapos[PM10]+2] = request->data_u16 >> 8;
                mfg_bthome_data[bthome_datapos[PM10]+1] = request->data_u16;
            }

            k_free(request);
    }
    if(numeltreceived == mfg_bthome_numelt) {
    //LOG_HEXDUMP_INF(mfg_bthome_data, mfg_bthome_sizetosend, "mfg_bthome_data");
    //printk("Send ADV Size %d\n", mfg_bthome_sizetosend);
                if(adv_activate == false)
                {
                    bt_ready();
                    adv_activate = true;
                } else {
     struct bt_data ad[] = {
        //BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
        BT_DATA(BT_DATA_SVC_DATA16, mfg_bthome_data, mfg_bthome_sizetosend),
    };
                    bt_le_adv_update_data(ad, ARRAY_SIZE(ad), NULL, 0);
                    printk("Update ADV\n");
                    //k_msleep(50);
                }
            }
  }
    }
    K_THREAD_DEFINE(publish_adv_thread_id, STACKSIZE, publish_adv_thread, NULL, NULL, NULL, PRIORITY, 0, 0);

    static void work_battery_handler(struct k_work *work)
    {
        struct bthome_adv_t bthome_adv;
        size_t size = sizeof(struct bthome_adv_t);
        bthome_adv.type = VOLTAGE;
        battery_setup();
        bthome_adv.data_u16 = (uint16_t) battery_sample();
        char *mem_ptr = k_malloc(size);
        memcpy(mem_ptr, &bthome_adv, size);
        k_fifo_put(&fifo_adv_data, mem_ptr);

        bthome_adv.type = BATTERY;
        bthome_adv.data_u8 = round(battery_level_pptt(bthome_adv.data_u16, battery_levels)/100);
        char *mem_ptr2 = k_malloc(size);
        memcpy(mem_ptr2, &bthome_adv, size);
        k_fifo_put(&fifo_adv_data, mem_ptr2);
        k_sem_give(&adv_send_ok);
        printk("Battery: %d mV (%d%%)\n",bthome_adv.data_u16, bthome_adv.data_u8);

        bthome_adv.type = VOLTAGEVDD;
        batteryvdd_setup();
        bthome_adv.data_u16 = (uint16_t) batteryvdd_sample();
        char *mem_ptr3 = k_malloc(size);
        memcpy(mem_ptr3, &bthome_adv, size);
        k_fifo_put(&fifo_adv_data, mem_ptr3);

        bthome_adv.type = BATTERYVDD;
        bthome_adv.data_u8 = round(battery_level_pptt(bthome_adv.data_u16, battery_nimh_levels)/100);
        char *mem_ptr4 = k_malloc(size);
        memcpy(mem_ptr4, &bthome_adv, size);
        k_fifo_put(&fifo_adv_data, mem_ptr4);
        k_sem_give(&adv_send_ok);
        printk("Battery VDD: %d mV (%d%%)\n",bthome_adv.data_u16, bthome_adv.data_u8);

    }
    K_WORK_DEFINE(work_battery, work_battery_handler);
void timer_battery_handler(struct k_timer *stm)
{
  k_work_submit(&work_battery);
}
    K_TIMER_DEFINE(timer_battery, timer_battery_handler, NULL);

    static void work_temperature_handler(struct k_work *work)
    {
        struct bthome_adv_t bthome_adv;
        sht4x_probe();
        bthome_adv.type = TEMPERATURE;
        bthome_adv.data_s16 = (int16_t)(sht4x_ReadTempeature()*100);
        size_t size = sizeof(struct bthome_adv_t);
        char *mem_ptr = k_malloc(size);
        memcpy(mem_ptr, &bthome_adv, size);
        k_fifo_put(&fifo_adv_data, mem_ptr);
        printk("Temperature: %d°\n",bthome_adv.data_s16);
        bthome_adv.type = HUMIDITY;
        bthome_adv.data_u8 = (uint8_t)(sht4x_ReadHumidity());
        char *mem_ptr2 = k_malloc(size);
        memcpy(mem_ptr2, &bthome_adv, size);
        k_fifo_put(&fifo_adv_data, mem_ptr2);
        k_sem_give(&adv_send_ok);
        printk("Humidity: %d pc \n",bthome_adv.data_u8);
    }
    K_WORK_DEFINE(work_temperature, work_temperature_handler);

void timer_temperature_handler(struct k_timer *stm)
{
    k_work_submit_to_queue(&workqueue_i2c, &work_temperature);
}
    K_TIMER_DEFINE(timer_temperature, timer_temperature_handler, NULL);

static void work_pressure_handler(struct k_work *work)
    {
        struct bthome_adv_t bthome_adv;
        bthome_adv.type = PRESSURE;
        bthome_adv.data_u32 = bmp180_readPressure();
        size_t size = sizeof(struct bthome_adv_t);
        char *mem_ptr = k_malloc(size);
        memcpy(mem_ptr, &bthome_adv, size);
        k_fifo_put(&fifo_adv_data, mem_ptr);
        k_sem_give(&adv_send_ok);
        printk("Pressure: %d Pa \n",bthome_adv.data_u32);
    }
    K_WORK_DEFINE(work_pressure, work_pressure_handler);


void timer_pressure_handler(struct k_timer *stm)
{
    k_work_submit_to_queue(&workqueue_i2c, &work_pressure);
}
    K_TIMER_DEFINE(timer_pressure, timer_pressure_handler, NULL);

static void work_pms_handler(struct k_work *work)
    {
  sds011_probe();
        struct bthome_adv_t bthome_adv;
            bthome_adv.type = PM25;
            bthome_adv.data_u16 = sds011_ReadPM25();
    size_t size = sizeof(struct bthome_adv_t);
            char *mem_ptr = k_malloc(size);
            memcpy(mem_ptr, &bthome_adv, size);
            k_fifo_put(&fifo_adv_data, mem_ptr);
            printk("PM25: %d\n",bthome_adv.data_u16);
            bthome_adv.type = PM10;
            bthome_adv.data_u16 = sds011_ReadPM10();
    char *mem_ptr2 = k_malloc(size);
            memcpy(mem_ptr2, &bthome_adv, size);
            k_fifo_put(&fifo_adv_data, mem_ptr2);
            printk("PM10: %d\n",bthome_adv.data_u16);
        k_sem_give(&adv_send_ok);
}
    K_WORK_DEFINE(work_pms, work_pms_handler);


void timer_pms_handler(struct k_timer *stm)
{
    k_work_submit_to_queue(&workqueue_pms, &work_pms);
}
    K_TIMER_DEFINE(timer_pms, timer_pms_handler, NULL);
   
    int main(void)
    {
        printk("Hello, It's TuxOutdooor\n");
        int err;

        //     bt_passkey_set(192847);
      mfg_bthome_data[0]=0xd2;
      mfg_bthome_data[1]=0xfc;
      mfg_bthome_data[2]=0x40;
      mfg_bthome_sizetosend=3;

      for(int i = 0; i < BTHOME_MAXELT; i++)
  {
    bthome_datapos[i] = 255;
  }

        err = bt_enable(NULL);
        if (err) {
            printk("Bluetooth init failed (err %d)\n", err);
            return 0;
        }


        const struct device *dev = DEVICE_DT_GET(GPIO_PORT);

        gpio_pin_configure(dev, 20, GPIO_OUTPUT);
        gpio_pin_set(dev, 20, 0);
        //bt_conn_auth_cb_register(&auth_cb);
        //static const struct device *const uart_dev = DEVICE_DT_GET(UART_DEVICE_NODE);
  
  
k_work_queue_init(&workqueue_i2c);
k_work_queue_start(&workqueue_i2c, stack_wq_i2c, K_THREAD_STACK_SIZEOF(stack_wq_i2c), PRIORITY, NULL);
k_work_queue_init(&workqueue_pms);
k_work_queue_start(&workqueue_pms, stack_wq_pms, K_THREAD_STACK_SIZEOF(stack_wq_pms), PRIORITY, NULL);


  if(sht4x_setup() == 0) { 
    printk("SHT4x enabled\n");
    k_work_submit_to_queue(&workqueue_i2c, &work_temperature);
    k_timer_start(&timer_temperature, K_SECONDS(300), K_SECONDS(300));
  eltenabled[TEMPERATURE] = 1;
  eltenabled[HUMIDITY] = 1;
  } else {
    printk("SHT4x disabled\n");
  }

  k_msleep(50);

  if(bmp180_setup() == 0) { 
    printk("BMP180 enabled\n");
    k_work_submit_to_queue(&workqueue_i2c, &work_pressure);
    k_timer_start(&timer_pressure, K_SECONDS(1800), K_SECONDS(1800));
  eltenabled[PRESSURE] = 1;
  } else {
    printk("BMP180 disabled\n");
  }

  k_msleep(50);
  if(sds011_setup() == 0) {
    printk("SDS011 enabled\n");
    k_work_submit_to_queue(&workqueue_pms, &work_pms);
  eltenabled[PM10] = 1;
  eltenabled[PM25] = 1;
  k_timer_start(&timer_pms, K_SECONDS(1800), K_SECONDS(1800));
  } else {
    printk("SDS011 disabled\n");
  }
 
  

  k_msleep(50);
 k_work_submit(&work_battery);
  k_timer_start(&timer_battery, K_SECONDS(3600), K_SECONDS(3600));
  eltenabled[BATTERY] = 1;
  eltenabled[VOLTAGE] = 1;
  eltenabled[BATTERYVDD] = 1;
  eltenabled[VOLTAGEVDD] = 1;

  mfg_bthome_build();

        k_sem_give(&ble_init_ok);
/*        for(;;)
        {
            if(findiftoprobe(BATTERY, true) < 5)
            {
                k_work_submit(&work_battery);
            }
            if(findiftoprobe(TEMPERATURE, true) < 5)
            {
                k_work_submit(&work_temperature);
            }
            if(findiftoprobe(PM25, true) < 5)
            {
                k_work_submit(&work_pms);
            }
            k_msleep(1000*findsmallestwait());

        } */

        /*int len = 0;
          int pm10_serial = 0;
          int pm25_serial = 0;
          int checksum_is;
          int checksum_ok = 0;
          int error = 1;
          float p10 = 0.0;
          float p25 = 0.0;
          for(;;)	{
          char value;
          if(uart_poll_in(uart_dev, &value) == 0)
          {
          printk("UART char received: 0x%x\n", value);
          printk("len %d\n", len);
          switch (len) {
          case (0): if (value != 170) { len = -1; }; break;
          case (1): if (value != 192) { len = -1; }; break;
          case (2): pm25_serial = value; checksum_is = value; break;
          case (3): pm25_serial += (value << 8); checksum_is += value; break;
          case (4): pm10_serial = value; checksum_is += value; break;
          case (5): pm10_serial += (value << 8); checksum_is += value; break;
          case (6): checksum_is += value; break;
          case (7): checksum_is += value; break;
          case (8): if (value == (checksum_is % 256)) { checksum_ok = 1; } else { len = -1; }; break;
          case (9): if (value != 171) { len = -1; }; break;
          }
          len++;
          if (len == 10 && checksum_ok == 1) {
          p10 = (float)pm10_serial/10.0;
          p25 = (float)pm25_serial/10.0;
          len = 0; checksum_ok = 0; pm10_serial = 0.0; pm25_serial = 0.0; checksum_is = 0;
          error = 0;
          printk("p10: %f p25: %f\n", p10, p25);
          }
          } else {
        //printk("No UART Data\n");
        k_msleep(100);
        }
        }*/

        return 0;
    }
